﻿//-----------------------------------------------------------------------
// <copyright file="ServerExtensionCSharpEmpty1.cs" company="Beckhoff Automation GmbH & Co. KG">
//     Copyright (c) Beckhoff Automation GmbH & Co. KG. All Rights Reserved.
// </copyright>
//-----------------------------------------------------------------------

using Newtonsoft.Json;
using System;
using System.IO;
using System.Reflection;
using System.Threading;
using TcHmiSrv.Core;
using TcHmiSrv.Core.General;
using TcHmiSrv.Core.Listeners;
using TcHmiSrv.Core.Tools.Management;

namespace ServerExtensionCSharpEmpty1
{
    // Represents the default type of the TwinCAT HMI server extension.
    public class ServerExtensionCSharpEmpty1 : IServerExtension
    {
        private readonly RequestListener requestListener = new RequestListener();
        private SQLConnecter sqlConnector;
        private object myLock = new object();
        private bool initDone = false;
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);



        /*
        GETTING STARTED

        The recommended way to get started is to look at a few of the sample extensions that are available on GitHub:
        https://github.com/Beckhoff/TF2000_Server_Samples

        The full documentation for the extension API can be found in the Beckhoff Information System:
        https://infosys.beckhoff.com/english.php?content=../content/1033/te2000_tc3_hmi_engineering/10591698827.html

        An offline version of this documentation is available at this path:
        %TWINCAT3DIR%..\Functions\TE2000-HMI-Engineering\Infrastructure\TcHmiServer\docs\TcHmiSrvExtNet.Core.Documentation.chm
        */

        // Called after the TwinCAT HMI server loaded the server extension.
        public ErrorValue Init()
        {
            this.requestListener.OnRequest += this.OnRequest;

            return ErrorValue.HMI_SUCCESS;
        }


        public static string AssemblyDirectory
        {
            get
            {
                string codeBase = Assembly.GetExecutingAssembly().CodeBase;
                UriBuilder uri = new UriBuilder(codeBase);
                string path = Uri.UnescapeDataString(uri.Path);
                return Path.GetDirectoryName(path);
            }
        }

        // Called when a client requests a symbol from the domain of the TwinCAT HMI server extension.
        private void OnRequest(object sender, TcHmiSrv.Core.Listeners.RequestListenerEventArgs.OnRequestEventArgs e)
        {
            try
            {
                e.Commands.Result = ServerExtensionCSharpEmpty1ErrorValue.ServerExtensionCSharpEmpty1Success;

                foreach (Command command in e.Commands)
                {
                    try
                    {
                        // Use the mapping to check which command is requested
                        switch (command.Mapping)
                        {

                            case "WriteValueInit":
                                {
                                    lock (myLock)
                                    {
                                        if (!initDone)
                                        {
                                            string s = AssemblyDirectory;
                                            string path1 = System.Reflection.Assembly.GetAssembly(typeof(ServerExtensionCSharpEmpty1)).Location;
                                            string directory = Path.GetDirectoryName(path1);
                                            var ss = System.IO.Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
                                            if (!log4net.LogManager.GetRepository().Configured)
                                            {
                                                s = "C:\\TwinCAT\\Functions\\TE2000-HMI-Engineering\\Infrastructure\\TcHmiServer\\Latest\\win-x86\\log4net.config";
                                                var configFile = new FileInfo(s);
                                                if (!configFile.Exists)
                                                {
                                                    throw new FileLoadException(String.Format("The configuration file {0} does not exist", configFile));
                                                }
                                                log4net.Config.XmlConfigurator.Configure(configFile);
                                            }
                                            sqlConnector = new SQLConnecter();
                                            sqlConnector.Do();
                                            initDone = true;
                                            log.Info("path1 =" + path1);
                                            log.Info("s =" + s);
                                            log.Info("ss =" + ss);
                                            log.InfoFormat("Initializing WriteValueInit {0}", command.WriteValue);
                                        }
                                    }
                                }
                                command.ExtensionResult = ServerExtensionCSharpEmpty1ErrorValue.ServerExtensionCSharpEmpty1Success;
                                command.ResultString = "Unknown command '" + command.Mapping + "' not handled.";
                                break;
                            case "WriteValueDelete":
                                {
                                    if(command.IsSet == true && command.WriteValue != null)
                                    {
                                        int index = Int32.Parse(command.WriteValue);
                                        sqlConnector.DeleteLine(index);

                                    }
                                }
                                command.ExtensionResult = ServerExtensionCSharpEmpty1ErrorValue.ServerExtensionCSharpEmpty1Success;
                                command.ResultString = "Unknown command '" + command.Mapping + "' not handled.";
                                break;
                            case "WriteValueAdd":
                                if (command.IsSet == true && command.WriteValue != null)
                                {
                                    Data routes_list = JsonConvert.DeserializeObject<Data>(command.WriteValue);
                                    sqlConnector.AddItem(routes_list.Test1, routes_list.Test2, routes_list.Test3, routes_list.ComboBox);
                                }
                                command.ExtensionResult = ServerExtensionCSharpEmpty1ErrorValue.ServerExtensionCSharpEmpty1Success;
                                command.ResultString = "Unknown command '" + command.Mapping + "' not handled.";
                                break;


                            case "GetDataValue":                          
                                command.ReadValue = SQLConnecter.MyData;
                                command.ExtensionResult = ServerExtensionCSharpEmpty1ErrorValue.ServerExtensionCSharpEmpty1Success;
                                command.ResultString = "Unknown command '" + command.Mapping + "' not handled.";
                                break;


                            default:
                                command.ExtensionResult = ServerExtensionCSharpEmpty1ErrorValue.ServerExtensionCSharpEmpty1Fail;
                                command.ResultString = "Unknown command '" + command.Mapping + "' not handled.";
                                break;
                        }
                    }
                    catch (Exception ex)
                    {
                        command.ExtensionResult = ServerExtensionCSharpEmpty1ErrorValue.ServerExtensionCSharpEmpty1Fail;
                        command.ResultString = "Calling command '" + command.Mapping + "' failed! Additional information: " + ex.ToString();
                    }
                }
            }
            catch (Exception ex)
            {
                throw new TcHmiException(ex.ToString(), ErrorValue.HMI_E_EXTENSION);
            }
        }
    }
}
